from __future__ import absolute_import, division, print_function

from .assemble import (
    get_package,
    Package,
)

__version__ = '0.1.6'

__package__ = 'assemble'
__title__ = 'Assemble'
__description__ = 'Assemble: Assemble the packages!'
__uri__ = 'https://github.com/CorverDevelopment/Assemble/'

__author__ = 'Nils Corver'
__email__ = 'nils@corverdevelopment.nl'

__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2016 Corver Development B.V.'

__all__ = [
    'get_package',
    'Package',
]
